@@ -10,7 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from six.moves import http_client
+from http import client as http_client
 
 from ironic_tempest_plugin.services.baremetal import base
 
@@ -20,8 +20,28 @@ class BaremetalClient(base.BaremetalClient):
     version = '1'
     uri_prefix = 'v1'
 
-    @staticmethod
-    def _get_headers(api_version):
+    node_attributes = (
+        'properties/cpu_arch',
+        'properties/cpus',
+        'properties/local_gb',
+        'properties/memory_mb',
+        'driver',
+        'instance_uuid',
+        'resource_class',
+        'protected',
+        'protected_reason',
+        # TODO(dtantsur): maintenance is set differently
+        # in newer API versions.
+        'maintenance',
+        'description',
+        'shard'
+    ) + tuple(
+        f'{iface}_interface'
+        for iface in base.SUPPORTED_INTERFACES
+    )
+
+    @classmethod
+    def _get_headers(cls, api_version):
         """Return headers for a request.
 
         Currently supports a header specifying the API version to use.
@@ -35,7 +55,7 @@ class BaremetalClient(base.BaremetalClient):
         headers = None
         if api_version is not None:
             extra_headers = True
-            headers = {'x-openstack-ironic-api-version': api_version}
+            headers = {cls.api_microversion_header_name: api_version}
         return extra_headers, headers
 
     @base.handle_errors
@@ -118,6 +138,11 @@ class BaremetalClient(base.BaremetalClient):
         """List all deploy templates."""
         return self._list_request('deploy_templates', **kwargs)
 
+    @base.handle_errors
+    def list_runbooks(self, **kwargs):
+        """List all runbooks."""
+        return self._list_request('runbooks', **kwargs)
+
     @base.handle_errors
     def show_node(self, uuid, api_version=None):
         """Gets a specific node.
@@ -247,6 +272,14 @@ class BaremetalClient(base.BaremetalClient):
         """
         return self._show_request('deploy_templates', deploy_template_ident)
 
+    def show_runbook(self, runbook_ident):
+        """Gets a specific runbook.
+
+        :param runbook_ident: Name or UUID of runbook.
+        :return: Serialized runbook as a dictionary.
+        """
+        return self._show_request('runbooks', runbook_ident)
+
     @base.handle_errors
     def create_node(self, chassis_id=None, **kwargs):
         """Create a baremetal node with the specified parameters.
@@ -261,7 +294,12 @@ class BaremetalClient(base.BaremetalClient):
 
         """
         node = {}
-        for field in ('resource_class', 'name', 'description'):
+        # Explicitly allow definition of network interface and deploy
+        # interface to allow tests to specify the required values
+        # as they hold a great deal of logic which is executed upon and
+        # they can ultimately impact test behavior.
+        for field in ('resource_class', 'name', 'description', 'shard',
+                      'network_interface', 'deploy_interface'):
             if kwargs.get(field):
                 node[field] = kwargs[field]
 
@@ -271,11 +309,16 @@ class BaremetalClient(base.BaremetalClient):
                             'cpus': kwargs.get('cpus', 8),
                             'local_gb': kwargs.get('local_gb', 1024),
                             'memory_mb': kwargs.get('memory_mb', 4096)},
-             'driver': kwargs.get('driver', 'fake')}
+             'driver': kwargs.get('driver', 'fake-hardware')}
         )
 
         return self._create_request('nodes', node)
 
+    @base.handle_errors
+    def create_node_raw(self, **kwargs):
+        """Create a baremetal node from the given body."""
+        return self._create_request('nodes', kwargs)
+
     @base.handle_errors
     def create_chassis(self, **kwargs):
         """Create a chassis with the specified parameters.
@@ -307,13 +350,12 @@ class BaremetalClient(base.BaremetalClient):
         :return: A tuple with the server response and the created port.
 
         """
-        port = {'extra': kwargs.get('extra', {'foo': 'bar'}),
-                'uuid': kwargs['uuid']}
+        port = {'extra': kwargs.get('extra', {'foo': 'bar'})}
 
         if node_id is not None:
             port['node_uuid'] = node_id
 
-        for key in ('address', 'physical_network', 'portgroup_uuid'):
+        for key in ('uuid', 'address', 'physical_network', 'portgroup_uuid'):
             if kwargs.get(key) is not None:
                 port[key] = kwargs[key]
 
@@ -414,6 +456,23 @@ class BaremetalClient(base.BaremetalClient):
         kwargs['name'] = name
         return self._create_request('deploy_templates', kwargs)
 
+    @base.handle_errors
+    def create_runbook(self, name, **kwargs):
+        """Create a runbook with the specified parameters.
+
+        :param name: The name of the runbook.
+        :param kwargs:
+            steps: steps of the runbook.
+            uuid: UUID of the runbook. Optional.
+            public: An optional boolean value indicating whether the runbook
+                is public (accessible to others)
+                or private (restricted to the owner).
+            extra: meta-data of the runbook. Optional.
+        :return: A tuple with the server response and the created runbook.
+        """
+        kwargs['name'] = name
+        return self._create_request('runbooks', kwargs)
+
     @base.handle_errors
     def delete_node(self, uuid):
         """Deletes a node having the specified UUID.
@@ -481,6 +540,15 @@ class BaremetalClient(base.BaremetalClient):
         """
         return self._delete_request('deploy_templates', deploy_template_ident)
 
+    @base.handle_errors
+    def delete_runbook(self, runbook_ident):
+        """Deletes a runbook having the specified name or UUID.
+
+        :param runbook_ident: Name or UUID of the runbook.
+        :return: A tuple with the server response and the response body.
+        """
+        return self._delete_request('runbooks', runbook_ident)
+
     @base.handle_errors
     def update_node(self, uuid, patch=None, **kwargs):
         """Update the specified node.
@@ -498,24 +566,8 @@ class BaremetalClient(base.BaremetalClient):
         else:
             params = {}
 
-        node_attributes = ('properties/cpu_arch',
-                           'properties/cpus',
-                           'properties/local_gb',
-                           'properties/memory_mb',
-                           'driver',
-                           'bios_interface',
-                           'deploy_interface',
-                           'rescue_interface',
-                           'instance_uuid',
-                           'resource_class',
-                           'protected',
-                           'protected_reason',
-                           # TODO(dtantsur): maintenance is set differently
-                           # in newer API versions.
-                           'maintenance',
-                           'description')
         if not patch:
-            patch = self._make_patch(node_attributes, **kwargs)
+            patch = self._make_patch(self.node_attributes, **kwargs)
 
         return self._patch_request('nodes', uuid, patch, params=params)
 
@@ -584,6 +636,18 @@ class BaremetalClient(base.BaremetalClient):
         return self._patch_request('deploy_templates', deploy_template_ident,
                                    patch)
 
+    @base.handle_errors
+    def update_runbook(self, runbook_ident, patch):
+        """Update the specified runbook.
+
+        :param runbook_ident: Name or UUID of the runbook.
+        :param patch: List of dicts representing json patches. Each dict
+            has keys 'path', 'op' and 'value'; to update a field.
+        :return: A tuple with the server response and the updated runbook.
+        """
+
+        return self._patch_request('runbooks', runbook_ident, patch)
+
     @base.handle_errors
     def set_node_power_state(self, node_uuid, state):
         """Set power state of the specified node.
@@ -596,9 +660,23 @@ class BaremetalClient(base.BaremetalClient):
         return self._put_request('nodes/%s/states/power' % node_uuid,
                                  target)
 
+    @base.handle_errors
+    def set_node_state(self, node_uuid, state, target):
+        """Set state for the specified node.
+
+        :param node_uuid: The unique identifier of the node.
+        :param state: The desired state to set.
+        :param target: The target state
+
+        """
+        target = {'target': target}
+        return self._put_request('nodes/%s/states/%s' % (node_uuid, state),
+                                 target)
+
     @base.handle_errors
     def set_node_provision_state(self, node_uuid, state, configdrive=None,
-                                 clean_steps=None, rescue_password=None):
+                                 clean_steps=None, rescue_password=None,
+                                 runbook=None):
         """Set provision state of the specified node.
 
         :param node_uuid: The unique identifier of the node.
@@ -608,6 +686,7 @@ class BaremetalClient(base.BaremetalClient):
             configuration drive string.
         :param clean_steps: A list with clean steps to execute.
         :param rescue_password: user password used to rescue.
+        :param runbook: The unique identifier of a runbook.
         """
         data = {'target': state}
         # NOTE (vsaienk0): Add both here if specified, do not check anything.
@@ -618,6 +697,8 @@ class BaremetalClient(base.BaremetalClient):
             data['clean_steps'] = clean_steps
         if rescue_password is not None:
             data['rescue_password'] = rescue_password
+        if runbook is not None:
+            data['runbook'] = runbook
         return self._put_request('nodes/%s/states/provision' % node_uuid,
                                  data)
 
@@ -675,6 +756,38 @@ class BaremetalClient(base.BaremetalClient):
         self.expected_success(http_client.OK, resp.status)
         return body
 
+    @base.handle_errors
+    def set_node_indicator_state(self, node_uuid, component, ind_ident, state):
+        """Get the current indicator state
+
+        :param node_uuid: The unique identifier of the node.
+        :param component: The Bare Metal node component.
+        :param ind_ident: The indicator of a Bare Metal component.
+        :param state: The state of an indicator of the component of the node.
+                        Possible values are: OFF, ON, BLINKING or UNKNOWN.
+
+        """
+        resp, body = self._put_request(
+            'nodes/%s/management/indicators/%s@%s'
+            % (node_uuid, ind_ident, component), {'state': state})
+        self.expected_success(http_client.OK, resp.status)
+        return resp, body
+
+    @base.handle_errors
+    def get_node_indicator_state(self, node_uuid, component, ind_ident):
+        """Get the current indicator state
+
+        :param node_uuid: The unique identifier of the node.
+        :param component: The Bare Metal node component.
+        :param ind_ident: The indicator of a Bare Metal component.
+
+        """
+        path = 'nodes/%s/management/indicators/%s@%s' % (node_uuid, ind_ident,
+                                                         component)
+        resp, body = self._list_request(path)
+        self.expected_success(http_client.OK, resp.status)
+        return resp, body
+
     @base.handle_errors
     def get_node_supported_boot_devices(self, node_uuid):
         """Get the supported boot devices of the specified node.
@@ -726,7 +839,7 @@ class BaremetalClient(base.BaremetalClient):
         headers = None
         if api_version is not None:
             extra_headers = True
-            headers = {'x-openstack-ironic-api-version': api_version}
+            headers = {self.api_microversion_header_name: api_version}
         return self._list_request('nodes/%s/vifs' % node_uuid,
                                   headers=headers,
                                   extra_headers=extra_headers)
@@ -859,3 +972,55 @@ class BaremetalClient(base.BaremetalClient):
 
         """
         return self._delete_request('allocations', allocation_ident)
+
+    @base.handle_errors
+    def list_node_history(self, node_uuid):
+        """List history entries for a node.
+
+        :param node_uuid: The unique identifier of the node.
+        """
+        return self._list_request('/nodes/%s/history' % node_uuid)
+
+    @base.handle_errors
+    def list_vendor_passthru_methods(self, node_uuid):
+        """List vendor-specific extensions (passthru) methods for a node
+
+        :param node_uuid: The unique identifier of the node.
+        """
+        return self._list_request('/nodes/%s/vendor_passthru/methods'
+                                  % node_uuid)
+
+    @base.handle_errors
+    def ipa_heartbeat(self, node_uuid, callback_url, agent_token,
+                      agent_version):
+        """Create a IPA heartbeat from the given body.
+
+        :param node_uuid: The unique identifier of the node.
+        :param callback_url: The URL of an active ironic-python-agent ramdisk
+        :param agent_token: The token of the ironic-python-agent ramdisk
+        :param agent_version: The version of the ironic-python-agent ramdisk
+        """
+        kwargs = {
+            'node_ident': node_uuid,
+            'callback_url': callback_url,
+            'agent_version': agent_version,
+            'agent_token': agent_token,
+        }
+
+        return self._create_request_no_response_body('heartbeat', kwargs)
+
+    @base.handle_errors
+    def show_inventory(self, uuid, api_version='1.81'):
+        """Gets hardware inventory for the specific node.
+
+        :param uuid: Unique identifier of the node in UUID format.
+        :param api_version: Ironic API version to use.
+        :return: Inventory as a dictionary.
+
+        """
+        extra_headers, headers = self._get_headers(api_version)
+        resp, body = self._show_request(
+            'inventory', uuid, headers=headers, extra_headers=extra_headers,
+            uri=f'{self.uri_prefix}/nodes/{uuid}/inventory')
+        self.expected_success(http_client.OK, resp.status)
+        return body
